<?php

namespace Framework\Foundation;

use Framework\Foundation\Config;
use Framework\Modules\UserAccess;
use Framework\Routes;
use Symfony\Component\HttpFoundation\Response;

abstract class HttpKernel
{

    public $request;
    protected $response;
    protected $middleware = [];
    protected $routes;

    public function __construct()
    {
        @session_set_cookie_params(0, '');
        if (!isset($_SESSION))
        {
            session_start();
        }

        $this->response = new Response;
    }

    /**
     * Handle request through router pipeline and prepare response
     * @param Symfony\Component\HttpFoundation\Request $request
     * @return Symfony\Component\HttpFoundation\Response
     */
    public function handle(\Symfony\Component\HttpFoundation\Request $request)
    {
        try
        {
            $this->request = $request;
            $this->bootstrappers();
            $this->response->setContent(
                    $this->sendRequestThroughRouter($this->request)
            );
        }
        catch (\Exception $e)
        {
            $exceptionHandler = new \Framework\Foundation\ExceptionHandler;
            $exceptionHandler->report($this->request, $e);
            $this->response->setContent(
                    $exceptionHandler->render($this->request, $e)
            );
        }

        return $this->response;
    }

    /**
     * Bootstap all runtime dependencies before any route hit
     * @return void
     */
    private function bootstrappers()
    {
        Application::bootstrap($this->request);
        Routes::$priorityList = Helper::getJsonFileData(BASE_PATH, 'routes.json');
    }

    /**
     * Send http request through a pipeline and middlewares, resolve controller and method name
     * Handle query and finally set content header based on response type
     * @param $request
     * @return object
     */
    private function sendRequestThroughRouter($request)
    {
        return (new Pipeline())->send($request)->through($this->middleware)->then(function($request)
                {
                    return $this->setContentHeader(
                                    $this->queryController(
                                            $this->resolveControllerMethodName($request)
                                    )
                    );
                });
    }

    /**
     * Resolve Controller and method name from URI path info
     * @param \Symfony\Component\HttpFoundation\Reques $request
     * @return string 
     */
    private function resolveControllerMethodName($request)
    {
        // http://localhost:8081/cms-v3/barebone/login
        $pathInfo  = $request->getPathInfo(); // /login
        $routePath = trim($pathInfo, '/'); // login
        $pathArr   = explode("/", $routePath); // Array ( [0] => login )
        // 1: Check routes.json routes if the request is not a ajax request
        if (!str_contains($routePath, "ajax/"))
        {

            if (Routes::$priorityList->has($routePath))
            {
                if (Routes::isProtected($routePath) && !UserAccess::validateToken())
                {
                    return $this->resolveProtectedRouteView($routePath);
                }

                Config::set('render_view', Routes::$priorityList[$routePath]['view']);
                return ['controller' => 'StaticPageController', 'method' => Routes::$priorityList[$routePath]['method']];
            }

            if (Routes::$priorityList->has($pathArr[0] . "/*"))
            {
                //Get the URL parameters
                $actual_path = $pathArr;
                unset($actual_path[0]);
                $actual_path = implode("/", $actual_path);
                $routePath   = $pathArr[0] . "/*";
                if (!file_exists($actual_path))
                {
                    if (Routes::isProtected($routePath) && !UserAccess::validateToken())
                    {
                        return $this->resolveProtectedRouteView($routePath);
                    }
                    Config::set('render_view', str_replace("*", $actual_path, Routes::$priorityList[$routePath]['view']));
                    return ['controller' => 'StaticPageController', 'method' => Routes::$priorityList[$routePath]['method']];
                }
                else
                {
                    //Directly serve dependency files (like css, images, js which are on the root level)
                    Config::set('render_view', $routePath);
                    return Routes::parseControllerMethod('/static-assets');
                }
            }
        }

        // 2: If there is no non ajax route mached into routes.json, then check into Routes::$list
        if (in_array($pathInfo, array_keys(Routes::$list)))
        {
            return Routes::parseControllerMethod($pathInfo);
        }

        // 3: Try to display by the page's name if everything fails
        Config::set('render_view', $routePath);
        return Routes::parseControllerMethod('/static-page');
    }

    /**
     * Check every restricted/protected routes which needs authentication
     * @param string $pathWithoutSlash
     * @return bool|string
     */
    private function resolveProtectedRouteView($pathWithoutSlash)
    {
        $controller = 'StaticPageController';
        $method     = 'staticPage';
        $view       = '404';

        if (!empty(Routes::$priorityList[Routes::$priorityList[$pathWithoutSlash]['redirect']]))
        {
            //If redirecting to another custom route
            $redirect = Routes::$priorityList[Routes::$priorityList[$pathWithoutSlash]['redirect']];
            $view     = $redirect['view'];
            $method   = $redirect['method'];
        }
        else if (!empty(Routes::$priorityList[$pathWithoutSlash]['redirect']))
        {
            //If redirecting to another static page
            $view = Routes::$priorityList[$pathWithoutSlash]['redirect'];
        }

        Config::set('render_view', $view);
        return compact('controller', 'method');
    }

    private function queryController($route)
    {
        $class = '\\Framework\\Controllers\\' . $route['controller'];

        if (!class_exists($class))
            throw new \Exception(sprintf('The %s class does\'t exists!', $route['controller']));

        $rawRequest = $this->request->getContent();
        if (!method_exists($route['method'], $class) && $route['controller'] == 'ExternalPlugin')
        {
            $class = '\\Extension\\' . $route['controller'];
        }
        if ($json = json_decode($rawRequest, true))
        {
            $this->request->json = $json;
        }

        $controllerObj = new $class;
        $result        = call_user_func_array(array($controllerObj, $route['method']), array($this->request));

        if (!$result && !method_exists($controllerObj, $route['method']))
        {
            throw new \Exception(sprintf("The `%s` method does not exists in the `%s` class.", $route['method'], $route['controller']));
        }
        return $result;
    }

    private function setContentHeader($response)
    {
        if (is_array($response) || is_object($response))
        {
            $this->response->headers->set('Content-Type', 'application/json');
            return json_encode($response);
        }

        return $response;
    }

}
