<?php

namespace Framework\Models;

use Illuminate\Support\Str;
use Framework\Foundation\Helper;
use Framework\Foundation\Config;

abstract class Model {

    protected $table, $connection, $connectionName=null, $databseConnection;

    protected $primary_key='id', $db_fields, $migrationLogFile;

    protected $ip;

    protected $tableSchema;

    public function __construct()
    {
        $modelName = $this->getModelName();
        $this->connectionName = Config::get(sprintf('database.models.%s.database_connections', $modelName));
        $this->table = Config::get(sprintf('database.models.%s.table', $modelName), $this->getTable());
        $this->primary_key = Config::get(sprintf('database.models.%s.primary_key', $modelName), 'id');
        $this->tableSchema = Config::get(sprintf('database.models.%s.table_schema', $modelName));
        
        $this->ip = Helper::getIP();
        $this->databseConnection = Helper::getDatabseConnection($this->connectionName);
        $this->connection();

        $this->migrationLogFile = sprintf("migrations_%s_%s_%s", $this->table, CMS_BAREBONE_VERSION, $this->databseConnection['driver']);
        $this->db_fields = $this->getTableColumns();
        // Add all the fields as property to the object so that "property_exists" can work with all fields
        foreach ($this->db_fields as $field) {
            if(empty($field)) continue;
            $this->{$field} = null;
        }
    }

    protected function getModelName() {
        return Str::lower(class_basename($this));
    }
    
    protected function connection() 
    {
        if (!$this->connection)
        {
            try
            {
                if (!defined('PDO::ATTR_DRIVER_NAME')) throw new \Exception("PDO not supported!");
                
                if($this->databseConnection['driver'] === 'mysql') {

                    $this->connection = new \PDO(
                        sprintf('mysql:dbname=%s;host=%s', $this->databseConnection['database'], $this->databseConnection['host']), $this->databseConnection['username'], $this->databseConnection['password']
                    );
                }

                if($this->databseConnection['driver'] === 'sqlite') { 
                    $this->connection = new \PDO(
                        sprintf("sqlite:%s/%s.sqlite", $this->SQLiteDbPath(), $this->databseConnection['database'])
                    );
                }
            }
            catch (\PDOException $e)
            {
                $this->modelEvent('error', $e->getMessage(), 0);
            }
            catch (\Exception $e) {
                $this->modelEvent('error', $e->getMessage(), 0);
            }
        }
    }

    protected function down() {

        return sprintf( "DROP TABLE IF EXISTS %s", $this->table);
    }

    protected function SQLiteDbPath()
    {
        $path = sprintf('%s%sSQLite', STORAGE_PATH, DS);
        if (!is_dir($path))
            mkdir($path, 0777, true);
        return $path;
    }

    protected function getTable() 
    {

        if($this->table) return $this->table;

        return str_replace(
            '\\', '', Str::snake(Str::plural(class_basename($this)))
        );
    }

    /**
     * @todo Close current connection
     */
    public function closeConn()
    {
        $this->connection = null;
    }

    protected function migrations() {

        return "";
    }

    public function createMigration() {
        try
        {
            if(!file_exists($this->SQLiteDbPath() .DS. $this->migrationLogFile)) {

                // If this constant is set to `true` it will delete all the tables from database
                if(FORCE_DB_MIGRATION) {
                    $this->connection->exec($this->down());
                }
                // This methos will run on every framwork version update but Will create table if not exists
                $this->connection->exec($this->migrations());
                Helper::createFileifNotExists($this->migrationLogFile, $this->SQLiteDbPath(), json_encode($this->getTableSchema()));
                $this->modelEvent('create_table', 'Created new '.$this->table.' table', 1);
            }

            // Ready 
            if(!$this->db_fields) {
                $this->db_fields = $this->getTableColumns();
            }

            return true;
        }
        catch (\PDOException $e)
        {
            $this->modelEvent('create_table', $e->getMessage(), 0);
            return false;
        }
    }

    protected function getTableColumns() {
        $data = Helper::getJsonFileData($this->SQLiteDbPath(), $this->migrationLogFile);
        
        return $data->pluck("Field")->all();
    }

    protected function getTableSchema() {

        if($this->databseConnection['driver'] === 'sqlite') {
            return $this->getTableSchemaFromSqlite();
        }

        return $this->getTableSchemaFromMysql();
    }

    protected function getTableSchemaFromMysql() {
        //Query MySQL with the PDO objecy.
        //The SQL statement is: DESCRIBE [INSERT TABLE NAME]
        $statement = $this->connection->query('DESCRIBE ' . $this->table);
        //Fetch our result.
        return $statement->fetchAll(\PDO::FETCH_ASSOC);
    }

    protected function getTableSchemaFromSqlite() {
        $statement = $this->connection->query("PRAGMA table_info({$this->table});");
        $unprepared = $statement->fetchAll(\PDO::FETCH_ASSOC);
        
        $schema = [];
        foreach($unprepared as $column) {

            $schema[] = [
                "Field" => array_get($column, 'name'),
                "Type" => array_get($column, 'type'),
                "Null" => array_get($column, 'notnull') ? "NO" : "YES",
                "Key" => array_get($column, 'pk') !== "0" ? "PRI" : null,
                "Default" => array_get($column, 'dflt_value'),
                "Extra" => ""
            ];
        }

        return $schema;
    }

    protected function instantiate($record) {
        $object = new static;

        foreach($record as $attribute=>$value){
		  if($object->has_attribute($attribute)) {
		    $object->$attribute = $value;
		  }
		}
		return $object;
	}
	
	private function has_attribute($attribute) {
	  // We don't care about the value, we just want to know if the key exists
	  // Will return true or false
	  return array_key_exists($attribute, $this->attributes());
	}

	protected function attributes() { 
		// return an array of attribute keys and their values
	  $attributes = array();
	  foreach ($this->db_fields as $field) {
	  	if (property_exists($this, $field)) {
	  		$attributes[$field] = $this->$field;
	  	}
	  }
	  return $attributes;
	}

    public function save() {
        // A new record won't have an id yet.
        return isset($this->{$this->primary_key}) ? $this->update() : $this->create();
    }
	
	public function create() {

	    $attributes = $this->attributes();

        $query = "INSERT INTO ".$this->table." (";
        $query .= join(", ", array_keys($attributes));
        $query .= ") VALUES (:";
        $query .= join(", :", array_keys($attributes));
        $query .= ")";
        $stmt = $this->connection->prepare($query);
    
        foreach($attributes as $key => $value) {
            $k = ':'.$key;
            $stmt->bindValue($k, $value);
        }
        if($this->has_attribute('created_at') && empty($attributes['created_at'])) {
            $stmt->bindValue(':created_at', time());
        }

        if ($stmt->execute())
        {
            $this->modelEvent('create', 'New record added', 1);
            return $this->connection->lastInsertId();
        }
        else
        {
            $this->modelEvent('create', 'New record added', 0);
            return false;
        }
	}

	public function update() {

        $attributes = $this->attributes();
        $attribute_pairs = array();
        foreach($attributes as $key => $value) {
            $attribute_pairs[] = "{$key}=:{$key}";
        }

        $query = "UPDATE ".$this->table." SET ";
        $query .= join(", ", $attribute_pairs);
        $query .= " WHERE id = ".$this->id;

        $stmt = $this->connection->prepare($query);

        foreach($attributes as $key => $value) {
            $k = ':'.$key;
            $stmt->bindValue($k, $value);
        }

        if($this->has_attribute('updated_at') && empty($attributes['updated_at'])) {
            $stmt->bindValue(':updated_at', time());
        }

        if ($stmt->execute())
        {
            $this->modelEvent('update', 'Record updated', 1);
            return $this->{$this->primary_key};
        }
        else
        {
            $this->modelEvent('update', 'Record updated', 0);
            return false;
        }
	}

    public static function find($queryArrayOrId) {

        return (new static)->findByParams($queryArrayOrId);
    }

    protected function findByParams($queryArrayOrId) {

        if(empty($queryArrayOrId)) return collect();

        $searchByPrimaryKey = false;

        if(!is_array($queryArrayOrId)) {
            $searchByPrimaryKey = true;
            $queryArrayOrId = [
                $this->primary_key => $queryArrayOrId
            ];
        }

        $params = collect($queryArrayOrId);
        $diffKeys = $params->diffKeys($this->attributes())->keys()->toArray();

        if(!empty($diffKeys)) {
            throw new \Exception(sprintf("Invaid column name `%s` provided for table %s table!", $diffKeys[0], $this->table));
        }

        $query = "SELECT * FROM ".$this->table." WHERE";
        foreach($params as $key => $val)  $query .= " {$key}=:{$key} AND";
        $query = trim($query, "AND ");

        $stmt  = $this->connection->prepare($query);

        $records = [];
        if ($stmt->execute($params->toArray()))
        {
            while ($row = $stmt->fetch()) $records[] = $this->instantiate($row);
        }

        if($searchByPrimaryKey) {
            // retrun single result
            return collect($records)->first();
        }

        return collect($records);
    }

	public function delete() {
		
		// Don't forget your SQL syntax and good habits:
		// - DELETE FROM table WHERE condition LIMIT 1
		// - escape all values to prevent SQL injection
		// - use LIMIT 1
        $query = "DELETE FROM ".$this->table;
        $query .= " WHERE id=:id";
        $query .= " LIMIT 1";

        $stmt = $this->connection->prepare($query);
        $stmt->bindValue(':id', $this->id);

        if($stmt->execute()) {

            $this->modelEvent('delete', sprintf('Record deleted (id: %s)', $this->id), 1);
            return true;
        }

        $this->modelEvent('delete', sprintf('Record deleted (id: %s)', $this->id), 0);
        return false;
	
		// NB: After deleting, the instance of User still 
		// exists, even though the database entry does not.
		// This can be useful, as in:
		//   echo $user->first_name . " was deleted";
		// but, for example, we can't call $user->update() 
		// after calling $user->delete().
	}

    private function modelEvent($event, $log, $success) {
        \Framework\Events\DatabaseEvent::dispatch([
            'type' => $event,
            'log' => $log,
            'success' => $success
        ]);
    }

    public static function firstOrCreate($queryArrayOrId) {

        $obj = new static;

        $result = $obj->findByParams($queryArrayOrId)->first();
        if(!empty($result->id)) return $result;
        
        if(!is_array($queryArrayOrId)) {
            $queryArrayOrId = [
                $obj->primary_key => $queryArrayOrId
            ];
        }

        $obj = $obj->instantiate($queryArrayOrId);
        $obj->{$obj->primary_key} = $obj->save();
        
        return $obj;
    }
}