<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;

class Limelight extends Crm implements CrmContracts
{

    protected $api_base_url;
    protected $username;
    protected $password;
    protected $campaignId;
    protected $shippingId;
    protected $forceGatewayId;
    protected $selectedProducts;
    protected $ipAddress;
    protected $notes;
    protected $httpClient;
    protected static $membershipMethods = [
        'order_update',
        'order_find',
        'order_view',
        'prospect_update',
        'prospect_find',
        'prospect_view',
        'get_alternative_provider',
        'coupon_validate',
        'order_update_recurring',
    ];
    private static $errorMessages = array(
        326   => 'URL is invalid',
        330   => 'No Status Passed',
        331   => 'Invalid Criteria',
        333   => 'No Orders Found',
        342   => 'Invalid Email Address',
        350   => 'Invalid order Id supplied',
        353   => 'Order already cancelled',
        354   => 'Cannot reset recurring',
        355   => 'Cannot start recurring',
        358   => 'Invalid Email Address. The Email Address is already being used by a customer',
        378   => 'Order already shipped',
        379   => 'Order is fully refunded/voided',
        380   => 'Order is not valid for force bill',
        381   => 'Customer is blacklisted',
        386   => 'Order has already been returned',
        413   => 'Product is not subscription based',
        421   => 'This order was placed on hold',
        422   => 'This order has not been sent to fulfillment yet',
        666   => 'User does not have permission to use this method',
        667   => 'This user account is currently disabled',
        668   => 'ip address is invalid',
        669   => 'Unauthorized to access campaign',
        700   => 'Invalid method supplied',
        1000  => 'SSL is required',
        1001  => 'General Error',
        4001  => 'Email not found for member',
        10303 => 'Invalid upsell product Id',
        10317 => 'Invalid email address',
        10321 => 'Invalid IP address',
        10333 => 'Order has been black listed',
        10600 => 'Invalid product Id',
        10666 => 'User does not have permission to use this method',
        11001 => 'General Error',
    );
    protected static $formKeys = [
        'NewProspect'          => [
            'address' => 'address1',
        ],
        'NewOrderWithProspect' => [
            'billingAddress' => 'billingAddress1',
            'cc_type'        => 'creditCardType',
            'cc_number'      => 'creditCardNumber',
            'cvv'            => 'CVV',
            'promo_code'     => 'promoCode',
        ],
        'NewOrder'             => [
            'address'        => 'shippingAddress1',
            'city'           => 'shippingCity',
            'country'        => 'shippingCountry',
            'state'          => 'shippingState',
            'zip'            => 'shippingZip',
            'billingAddress' => 'billingAddress1',
            'cc_type'        => 'creditCardType',
            'cc_number'      => 'creditCardNumber',
            'cvv'            => 'CVV',
            'promo_code'     => 'promoCode',
        ],
    ];

    public function __construct()
    {
        parent::__construct();
    }

    protected function prepareEndpointApi($api_base_url, $method)
    {
        $endpointUrl = $api_base_url . sprintf('/admin/%s.php', in_array($method, self::$membershipMethods) ? 'membership' : 'transact');
        return $endpointUrl;
    }

    protected function httpRequest($api, $new_form_params)
    {
        $this->httpClient->setDefaultOption('verify', false);

        $form_params = [
            'username' => $this->username,
            'password' => $this->password,
        ];

        $form_params = array_merge_recursive($form_params, $new_form_params);
        $form_params = array_filter($form_params);

        $response = $this->httpClient->post($api, [
            'body' => $form_params,
        ]);

        return $response;
    }

    protected function callApi($method, $fromData)
    {
        $api = $this->prepareEndpointApi($this->api_base_url, $method);
        $fromData['method'] = $method;
        if(array_key_exists('creditCardType',$fromData) && $fromData['creditCardType'] == 'paypal'){
            

            if(strpos($_SERVER['HTTP_HOST'], 'www.') !== false){
                $url = Config::get('funnel.url');
                $url = str_replace('https://','https://www.',$url);
            }
            $fromData['alt_pay_return_url'] = $url . '/paypal-response';            

        }
        if (Config::get('addons.3d_validation.enabled'))
        {
            $fromData['three_d_redirect_url'] = Config::get('funnel.url');
        } 
        $response = $this->httpRequest($api, $fromData);
        if ($response->getStatusCode() == 200)
        {
            $apiResponse = $response->getBody()->getContents();
            parse_str($apiResponse, $apiResponseArr);

            ;

            if($this->isPaypalExpressResponse($apiResponse)) {
                return [
                    'status' => true, 
                    'api_response' => [],
                    'is_html' => true,
                    'isPaypalExpressReponse' => 1, 
                    'html_data' => $apiResponse
                ];              
            }            

            $apiResponseCode = array_get($apiResponseArr, 'response_code', array_get($apiResponseArr, 'responseCode'));
            $apiResponseArr['response_message'] = array_get($apiResponseArr, 'error_message', array_get($apiResponseArr, 'errorMessage', ''));

            if ($apiResponseCode == 100)
            {
                if ($method == 'NewOrder') $apiResponseArr['prospectId'] = rand(1111, 9999);
                return ['status' => true, 'api_response' => $apiResponseArr];
            }
            
            if ($apiResponseCode == 800)
            {
                return ['status' => true, 'api_response' => $apiResponseArr['declineReason'], 'isOrderDecline' => true];
            }
            
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
        
        return ['status' => false, 'api_response' => ['response_message' => 'The request couldn\'t be completed! Please try again later']];
    }

    public function newProspect($formData)
    {
        $method     = 'NewProspect';
        $configData = [];

        $configData['campaignId'] = $this->campaignId;
        $configData['ipAddress']  = $this->ipAddress;

        $crmData = array_merge_recursive($formData, $configData);
        $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData = array_filter($crmData);

        if (Config::isSinglePurchaseEnable() && $this->checkAlreadyPurchased($crmData))
        {
            $message = Config::get('addons.scheduled-blocker.display_message');
            return ['status' => false, 'api_response' => ['response_message' => $message]];
        }

        return $this->callApi($method, $crmData);
    }

    public function newOrderWithProspect($checkoutData)
    {
        $method     = 'NewOrderWithProspect';
        $configData = [];
        $expirationDate = '';

        if (!empty($checkoutData['expmonth']) && !empty($checkoutData['expyear']))
        {
            $expirationDate = $checkoutData['expmonth'] . $checkoutData['expyear'];
            unset($checkoutData['expmonth']);
            unset($checkoutData['expyear']);
        }
       
        if ( Config::isSinglePurchaseEnable() && $this->checkAlreadyPurchased($checkoutData))
        {
            $message = Config::get('addons.scheduled-blocker.display_message');
            return ['status' => false, 'api_response' => ['response_message' => $message]];
        }

        if ($this->isBillingProfileEnable())
        {
            $this->builtBillingProductArray($configData);
        }
        else
        {
            $this->nonBillingModelOrderData($checkoutData);
        }

        $configData['expirationDate'] = $expirationDate;
        $crmData = $this->generateRuntimeCrmPayload($configData, $checkoutData, $method);
        return $this->callApi($method, $crmData);
    }

    public function newOrder($orderData, $configData, $type)
    {
        $method = 'NewOrder';

        if(isset($orderData['creditCardType']) && $orderData['creditCardType'] != 'offline'){
            unset($orderData['creditCardType']);
        }

        if (!empty($orderData['expmonth']) && !empty($orderData['expyear']))
        {
            $expirationDate = $orderData['expmonth'] . $orderData['expyear'];
            unset($orderData['expmonth']);
            unset($orderData['expyear']);
        }
        else
        {
            $expirationDate = '';
        }

        $orderData['expirationDate'] = $expirationDate;

        if ($type === 'SplitOrder' || $type === 'UpsellOrder')
        {
            $this->selectedProducts = $configData['selectedProducts'];
            $this->campaignId       = $configData['campaignId'];
        }

        if (Config::isSinglePurchaseEnable() && $this->checkAlreadyPurchased($orderData))
        {
            $message = Config::get('addons.scheduled-blocker.display_message');
            return ['status' => false, 'api_response' => ['response_message' => $message]];
        }

        if ($this->isBillingProfileEnable())
        {
            $this->builtBillingProductArray($orderData);
        }
        else
        {
            $this->nonBillingModelOrderData($orderData);
        }

        unset($configData['selectedProducts']);
        $configData['ipAddress'] = $this->ipAddress;
        $crmData = $this->generateRuntimeCrmPayload($configData, $orderData, $method);
        return $this->callApi($method, $crmData);
    }

    private function isPaypalExpressResponse($apiResponse){
        if(preg_match("/paypal/i",$apiResponse)){
           return true; 
        }
        return false;
    }

    private function generateRuntimeCrmPayload($configData, $orderData, $method='NewOrderWithProspect') {
        $configData['campaignId']     = $this->campaignId;
        $configData['shippingId']     = $this->shippingProfile->get('shipping_profile_crm_id');
        $configData['tranType']       = 'Sale';

        if (!empty($this->notes))
        {
            if ($this->skip_website_url)
            {
                $configData['notes'] = $this->notes;
            }
            else
            {
                $configData['notes'] = $this->notes . ' | ' . 'site : ' . $orderData['current_url'];
            }
        }
        else
        {
            if ($this->skip_website_url)
            {
                $configData['notes'] = '';
            }
            else
            {
                $configData['notes'] = 'site : ' . $orderData['current_url'];
            }
        }

        if (!empty($_SESSION['http_url_referer']))
        {
            $configData['notes'] .= ' | ' . 'referer : ' . $_SESSION['http_url_referer'];
        }

        if($method == 'NewOrder') {
            unset($configData['selectedProducts']);
            if (isset($configData['forceGatewayId']) && $configData['forceGatewayId'] > 0)
            {
                $configData['preserve_force_gateway'] = 1;
            }
            else
            {
                unset($configData['forceGatewayId']);
            }
        }

        if($method == 'NewOrderWithProspect') {
            if ($this->forceGatewayId > 0)
            {
                $configData['forceGatewayId']         = $this->forceGatewayId;
                $configData['preserve_force_gateway'] = 1;
            }
        }

        $crmData = array_merge_recursive($orderData, $configData);
        $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData = array_filter($crmData);

        if (Config::get('addons.3d_validation.enabled') && Config::get('payment_provider.ll_3ds_gateway_id'))
        {
            $this->perform3DSActions($method, $crmData, $orderData);
        }

        if ($this->isConsent($crmData))
        {
            $crmData['consent_required'] = 1;
        }

        return $crmData;
    }

    private function nonBillingModelOrderData(&$orderData) {
        foreach ($this->selectedProducts as $selectedProduct)
        {
            $productIdArr[] = $selectedProduct['crm_product_id'];
            $orderData['product_qty_' . $selectedProduct['crm_product_id']] = $selectedProduct['crm_product_qty'];
            $orderData['dynamic_product_price_' . $selectedProduct['crm_product_id']] = $selectedProduct['crm_product_price'];
        }
    
        if (count($this->selectedProducts) > 1)
        {
            $orderData['upsellCount']      = 1;
            $orderData['upsellProductIds'] = '';
            $count                         = 0;
            foreach ($productIdArr as $key => $val)
            {
                if ($count == 0)
                {
                    $count++;
                    continue;
                }
                if ($count == 1)
                {
                    $orderData['upsellProductIds'] = $val;
                    $count++;
                    continue;
                }
                $orderData['upsellProductIds'] .= ',' . $val;
                $count++;
            }
        }
    
        $orderData['productId'] = implode(",", $productIdArr);
    }

    public function orderView($queryStrings, $orderIds)
    {
        $method              = 'order_view';
        $crmData['order_id'] = implode(',', $orderIds);

        $response = $this->callApi($method, $crmData);

        if (!empty($response['api_response']['data']))
        {
            $apiResponseArr = json_decode($response['api_response']['data'], true);
        }
        else
        {
            $apiResponseArr = $response['api_response'];
        }

        return $apiResponseArr;
    }

    public function validateCoupon($couponData)
    {
        $method     = 'coupon_validate';
        $configData = [];

        foreach ($this->selectedProducts as $selectedProduct)
        {
            $selectedProduct['crm_product_id'] = trim($selectedProduct['crm_product_id']);
            $productIdArr[] = $selectedProduct['crm_product_id'];
            $couponData[sprintf("product_qty_%s", $selectedProduct['crm_product_id'])] = $selectedProduct['crm_product_qty'];
        }

        $couponData['product_ids'] = implode(",", $productIdArr);

        $configData['campaign_id'] = $this->campaignId;
        $configData['shipping_id'] = $this->shippingProfile->get('shipping_profile_crm_id');

        $crmData = array_merge_recursive($couponData, $configData);
        $crmData = array_filter($crmData);

        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function remoteUpsell($checkoutData)
    {
        $method = 'NewOrderCardOnFile';

        $crmData  = $checkoutData;
        $crmData  = array_filter($crmData);
        $response = $this->callApi($method, $crmData);

        return $response;
    }

    public function checkout3DValidation($validationData = [])
    {
        try
        {
            $api          = $this->api_base_url . '/admin/alternative_provider/limelight_3d_verify.php';
            $response     = $this->httpRequest($api, $validationData);
            $httpResponse = $response->getStatusCode();
            if ($httpResponse == 200)
            {
                $apiResponse = $response->getBody()->getContents();
                return $apiResponse;
            }
            throw new \Exception('Something went wrong while processing the request');
        }
        catch (\Exception $ex)
        {
            throw new \Exception($ex->getMessage());
        }
    }

    private function perform3DSActions($method, $crmData, $formData)
    {
        $_SESSION['3ds_meta'] = array(
            'current_url' => $formData['current_url'], 'method'      => $method,
            'next_url'    => $formData['next_url'], 'crm_data'    => $crmData,
        );

        $priceFields = preg_grep('/_price/', array_keys($crmData));
        $totalAmount = 0.00;
        foreach ($priceFields as $priceField)
        {
            $totalAmount += (float) $crmData[$priceField];
        }

        $params = array(
            'token'        => $this->get3DAuthToken(),
            'gateway_id'   => Config::get('offer_details.step1.crm_force_gateway_id'),
            'x_relay_url'  => sprintf(
                    '%s://%s/limelight-3d-response?%s', app()->container()->get('scheme'), $this->getBaseUrl(), http_build_query(array(
                'prospect_id' => 12345,
                    ))
            ),
            'x_card_num'   => $crmData['creditCardNumber'],
            'x_exp_month'  => substr($crmData['expirationDate'], 0, 2),
            'x_exp_year'   => substr($crmData['expirationDate'], 2),
            'total_amount' => number_format($totalAmount, 2),
        );

        $url = sprintf(
                '%s/admin/alternative_provider/limelight_3d_verify.php', $this->api_base_url
        );

        header('Content-Type: application/json');
        echo json_encode(array(
            'status'    => 'success', 'is_html'   => true,
            'html_data' => $this->generate3DSForm($url, $params),
        ));
        exit;
    }

    public function getBaseUrl()
    {
        $currentPath = $_SERVER['PHP_SELF'];
        $pathInfo    = pathinfo($currentPath);
        $path        = trim($pathInfo['dirname'], '/');
        $hostName    = trim($_SERVER['HTTP_HOST'], '/');
        return rtrim($hostName . '/' . $path, '/');
    }

    public function get3DAuthToken()
    {
        $url = sprintf(
                '%s/admin/api/api_token.php', $this->api_base_url
        );

        $response = $this->httpClient->post($url, [
            'body' => array(
                'username' => $this->username,
                'password' => $this->password,
            ),
        ]);

        if ($response->getStatusCode() == 200)
        {
            parse_str($response->getBody()->getContents(), $data);
            if (!empty($data['token']))
            {
                return $data['token'];
            }
        }

        throw new \Exception('Something went wrong while processing the request');
    }

    protected function generate3DSForm($url, $data)
    {
        $form = <<<HTML
            <html>
                <head></head>
                <body onload="document.threeDForm.submit()" >
                    <form name="threeDForm" action="{$url}" id="threeDForm" method="POST">
HTML;
        foreach ($data as $key => $value)
        {
            $form .= <<<HTML
                        <input type="hidden" name="{$key}" value="{$value}">
HTML;
        }
        $form .= <<<HTML
                    </form>
                </body>
            </html>
HTML;

        return $form;
    }

    public function process3DOrder($method, $crmData)
    {
        return $this->callApi($method, $crmData);
    }

    public function isDecline($response)
    {
        if(!empty($response['isOrderDecline'])) return true;
        return !empty($response['responseCode']) && in_array($response['responseCode'], [800]) ? true : false;
    }

    public function singleOrderView($formData)
    {
        $method           = 'order_view';
        $data['order_id'] = $formData['orderId'];
        $response         = $this->callApi($method, $data);
        if (!empty($response['status']) //status
                && ((!empty($response['api_response']['response_code']) && ($response['api_response']['response_code'] == 100)) //api_response
                || (!empty($response['apiResponse']['response_code']) && ($response['apiResponse']['response_code'] == 100)))
        )
        {
            $key = array_key_exists('api_response', $response) ? 'api_response' : 'apiResponse';

            $placeholder['order_id']           = !empty($response[$key]['order_id']) ? $response[$key]['order_id'] : '';
            $placeholder['email']              = !empty($response[$key]['email_address']) ? $response[$key]['email_address'] : '';
            $placeholder['billing_first_name'] = !empty($response[$key]['billing_first_name']) ? $response[$key]['billing_first_name'] : '';
            $placeholder['billing_last_name']  = !empty($response[$key]['billing_last_name']) ? $response[$key]['billing_last_name'] : '';

            return ['status' => 'success', 'apiResponse' => $response[$key], 'placeholderToken' => $placeholder];
        }
        $key = array_key_exists('api_response', $response) ? 'api_response' : 'apiResponse';
        return ['status' => 'error', 'apiResponse' => !empty(self::$errorMessages[$response[$key]['response_code']]) ? self::$errorMessages[$response[$key]['response_code']] : 'Something went wrong.'];
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        return (array_key_exists("order_id", $data['apiResponse']) && $data['apiResponse']['email_address'] == strtolower($formData['email'])) ? TRUE : FALSE;
    }

    public function prepareDataForCancel($data, $formData)
    {
        $formData['order_id'] = $data['apiResponse']['order_id'];
        return $formData;
    }

    public function cancelOrder($formData)
    {
        $method           = 'order_update_recurring';
        $data['order_id'] = $formData['orderId'];
        $data['status']   = 'stop';
        $response         = $this->callApi($method, $data);
        if (!empty($response['status']) //status
                && ((!empty($response['api_response']['response_code']) && ($response['api_response']['response_code'] == 100)) //api_response
                || (!empty($response['apiResponse']['response_code']) && ($response['apiResponse']['response_code'] == 100)))
        )
        {
            return ['status' => 'success', 'apiResponse' => 'Order cancelled successfully.'];
        }
        $key = array_key_exists('api_response', $response) ? 'api_response' : 'apiResponse';
        return ['status' => 'error', 'apiResponse' => !empty(self::$errorMessages[$response[$key]['response_code']]) ? self::$errorMessages[$response[$key]['response_code']] : 'Something went wrong.'];
    }

    /**
     * Check billing modal is enabled or not if yes then set the value
     * @return boolean
     */
    private function isBillingProfileEnable()
    {
        $flag = false;

        foreach ($this->selectedProducts as $selectedProduct)
        {
            if (!empty($selectedProduct['offer_id']) && !empty($selectedProduct['billing_model_id']))
            {
                $flag = true;
                break;
            }
        }
        return $flag;
    }

    /**
     * Method to build product array if Billing Modal is enabled.
     * 
     * @param array $orderData Order Array which will be passed into LL
     */
    private function builtBillingProductArray(&$orderData)
    {
        foreach ($this->selectedProducts as $selectedProduct)
        {
            $payload = [
                'offer_id'         => $selectedProduct['offer_id'],
                'billing_model_id' => $selectedProduct['billing_model_id'],
                'product_id'       => $selectedProduct['crm_product_id'],
                'quantity'         => $selectedProduct['crm_product_qty'],
                'price'            => $selectedProduct['crm_product_price'],
            ];

            if (!empty($selectedProduct['trial_product_id']))
            {
                $payload['trial'] = [
                    'product_id' => $selectedProduct['trial_product_id'],
                ];
            }

            $orderData['offers'][] = $payload;
        }
    }

    /**
     * Checking order exist for an User using Card or Email
     * @param type $orderData
     * @return type Boolean
     */
    private function checkAlreadyPurchased($orderData)
    {
        $method            = 'order_find';
        $userBlockSettings = Config::get('addons.scheduled-blocker');

        if($userBlockSettings['type'] === 'time') {
            $startDate  = date('m/d/Y', strtotime('-' . $userBlockSettings['type_value'] . 'Minutes'));
            $endDate   = date('m/d/Y');
        }
        if($userBlockSettings['type'] === 'date') {
            $startDate  = '01/01/2015';
            $endDate = date('m/d/Y', strtotime($userBlockSettings['type_value']));
        }

        if($startDate > $endDate) return false;

        $curlPostData          = [
            'method'      => $method,
            'campaign_id' => 'all',
            'start_date'  => $startDate,
            'end_date'    => $endDate,
            'search_type' => 'all',
            'return_type' => 'order_view',
        ];
        $curlPostData_declines = $curlPostData;

        if (!empty($userBlockSettings['block_type']) && in_array('credit_card', $userBlockSettings['block_type']) && !empty($orderData['cc_number']))
        {
            $creditCardNumber  = $orderData['cc_number'];
            $first_6_cc        = substr($creditCardNumber, 0, 6);
            $last_4_cc         = substr($creditCardNumber, -4);
            $criteria          = "'first_6_cc=" . $first_6_cc . ",last_4_cc=" . $last_4_cc . ",all'";
            $criteria_declines = "'first_6_cc=" . $first_6_cc . ",last_4_cc=" . $last_4_cc . ",declines'";
        }
        elseif (!empty($orderData['email']) && (empty($userBlockSettings['block_type']) || in_array('email', $userBlockSettings['block_type'])))
        {
            $criteria          = "'email=" . $orderData['email'] . ",all'";
            $criteria_declines = "'email=" . $orderData['email'] . ",declines'";
        }
        elseif (!empty($userBlockSettings['block_type']) && in_array('ip', $userBlockSettings['block_type']))
        {
            $criteria  = "'ip_address=" . (!empty($orderData['ipAddress']) ? $orderData['ipAddress'] : $this->ipAddress) . ",all'";
            $criteria_declines = "'ip_address=" . (!empty($orderData['ipAddress']) ? $orderData['ipAddress'] : $this->ipAddress) . ",declines'";
        }
        else
        {
            return false;
        }

        $curlPostData['criteria']          = $criteria;
        $curlPostData_declines['criteria'] = $criteria_declines;

        $response_all = $this->callApi($method, $curlPostData);
        $allOrderIds  = 0;
        if (!empty($response_all['api_response']['response_code']) && $response_all['api_response']['response_code'] == 100)
        {
            $allOrderIds = $response_all['api_response']['total_orders'];
        }

        $response_declines = $this->callApi($method, $curlPostData_declines);
        $allDeclines       = 0;
        if (!empty($response_declines['api_response']['response_code']) && $response_declines['api_response']['response_code'] == 100)
        {
            $allDeclines = $response_declines['api_response']['total_orders'];
        }

        return ($allOrderIds > $allDeclines) ? true : false;
    }

    /**
     * Check if CRM is limelight and consent_notification flag is true.
     * @return boolean 
     */
    private function isConsent($crmData)
    {
        $apiType             = Config::get('payment_provider.api_type');
        $consentNotification = Config::get('consent_notification');
        return ($apiType == 'limelight' && $consentNotification && in_array($crmData['creditCardType'], ['master', 'mastercard'])) ? true : false;
    }

}
