<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use GuzzleHttp\Client;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;

class Mbilling extends Crm implements CrmContracts
{

    protected $api_base_url;
    protected $username;
    protected $password;
    protected $campaignId;
    protected $shippingId;
    protected $selectedProducts;
    protected $httpClient;
    protected $ipAddress;
    protected static $formKeys = [
        'NewProspect'          => [
            'address'    => 'address1',
            'zip'        => 'zipcode',
            'firstName'  => 'first_name',
            'lastName'   => 'last_name',
            'campaignId' => 'campaign_id',
        ],
        'NewOrderWithProspect' => [
            'cc_type'          => 'card_type_id',
            'cc_number'        => 'card_number',
            'cvv'              => 'card_cvv',
            'expmonth'         => 'card_exp_month',
            'expyear'          => 'card_exp_year',
            'prospectId'       => 'customer_id',
            'campaignId'       => 'campaign_id',
            'billingFirstName' => 'ship_fname',
            'billingLastName'  => 'ship_lname',
            'billingCountry'   => 'ship_country',
            'billingAddress'   => 'ship_address1',
            'billingCity'      => 'ship_city',
            'billingState'     => 'ship_state',
            'billingZip'       => 'ship_zipcode',
        ],
        'NewOrder'             => [
            'cc_type'          => 'card_type_id',
            'cc_number'        => 'card_number',
            'cvv'              => 'card_cvv',
            'expmonth'         => 'card_exp_month',
            'expyear'          => 'card_exp_year',
            'prospectId'       => 'customer_id',
            'campaignId'       => 'campaign_id',
            'billingFirstName' => 'ship_fname',
            'billingLastName'  => 'ship_lname',
            'billingCountry'   => 'ship_country',
            'billingAddress'   => 'ship_address1',
            'billingCity'      => 'ship_city',
            'billingState'     => 'ship_state',
            'billingZip'       => 'ship_zipcode',
        ],
    ];

    public function __construct()
    {
        parent::__construct();
        $this->is3DValidationEnabled = Config::get('addons.3d_validation.enabled');
        $this->offer_id              = Config::get('payment_provider.m1_offer_id');
        $this->httpClient            = new Client([
            'strict'          => false,
            'protocols'       => ['http', 'https'],
            'allow_redirects' => true,
            'headers'         => ['content-type' => 'application/json'],
            'defaults'        => ['exceptions' => false, 'verify' => false],
        ]);
    }

    protected function cardTypeId($key)
    {
        $cardTypeID = ['master' => 1, 'mastercard' => 1, 'visa' => 2];
        if (array_key_exists($key, $cardTypeID))
        {
            return $cardTypeID[$key];
        }
        else
        {
            throw new \Exception("Invalid card type : {$key}", 9999);
        }
    }

    protected function prepareEndpointApi($api_base_url, $method)
    {
        $methodSlugMapping = [
            'NewProspect'          => 'api/customer/doAddCustomerAddress',
            'NewOrderWithProspect' => 'api/order/doAddProcess',
            'NewOrder'             => 'api/order/doAddProcess',
            'orderView'            => 'api/order/view/',
            'process3ds'           => 'api/transaction/doProcess3ds',
        ];
        if (array_key_exists($method, $methodSlugMapping))
        {
            $endpointUrl = $api_base_url . '/' . $methodSlugMapping[$method];
        }
        else
        {
            throw new \Exception("Invalid method name : {$method}", 9999);
        }
        return $endpointUrl;
    }

    protected function httpRequest($api, $new_form_params)
    {
        // Using Deconstructor to solve cURL error 2 , we need to do R&D for this
        $this->httpClient = null;
        $this->httpClient = new Client();
        $this->httpClient->setDefaultOption('verify', false);

        $form_params = [
            'user_id'       => $this->username,
            'user_password' => urlencode($this->password),
        ];
        $form_params = array_merge_recursive($form_params, $new_form_params);
        $form_params = array_filter($form_params);
        $response    = $this->httpClient->post($api, ['body' => $form_params]);
        return $response;
    }

    protected function callApi($method, $form_post_params)
    {
        $apiResponseArr             = [];
        $api                        = $this->prepareEndpointApi($this->api_base_url, $method);
        $form_post_params['method'] = $method;
        if ($this->is3DValidationEnabled)
        {
            $form_post_params['3ds_redirect_url'] = Config::get('funnel.url') . '/mbilling-3ds-response';
        }
        try
        {
            $response     = $this->httpRequest($api, $form_post_params);
            $httpResponse = $response->getStatusCode();
            $apiResponse  = json_decode($response->getBody()->getContents(), true);
            switch ($httpResponse)
            {
                case 200:
                    if (!empty($apiResponse['data']['transaction']['response_code']) && $apiResponse['data']['transaction']['response_code'] == 101)
                    {
                        $this->perform3DSActions($method, $apiResponse, $form_post_params);
                    }
                    else
                    {
                        $apiResponseArr['response_message'] = $apiResponse['message'];
                        if ($method == 'NewProspect')
                        {
                            $apiResponseArr['prospectId']        = !empty($apiResponse['data']['customer']) ? $apiResponse['data']['customer'] : null;
                            $apiResponseArr['customerAddressId'] = !empty($apiResponse['data']['customer_address_id']) ? $apiResponse['data']['customer_address_id'] : null;
                        }
                        if ($method == 'NewOrderWithProspect' || $method == 'NewOrder')
                        {
                            $apiResponseArr = [
                                'orderId'    => !empty($apiResponse['data']['transaction']['order_id']) ? $apiResponse['data']['transaction']['order_id'] : null,
                                'customerId' => !empty($apiResponse['data']['transaction']['customer_id']) ? $apiResponse['data']['transaction']['customer_id'] : null,
                                'prospectId' => !empty($apiResponse['data']['transaction']['customer_id']) ? $apiResponse['data']['transaction']['customer_id'] : null,
                                'gatewayId'  => !empty($apiResponse['data']['transaction']['merchant_id']) ? $apiResponse['data']['transaction']['merchant_id'] : null,
                            ];
                        }
                        return ['status' => $apiResponse['success'], 'api_response' => $apiResponseArr];
                    }
                    break;

                case 400:
                    $apiResponseArr = [
                        'response_message' => !empty($apiResponse['message']) ? $apiResponse['message'] : null,
                        'data'             => !empty($apiResponse['data']) ? $apiResponse['data'] : null
                    ];
                    return ['status' => false, 'api_response' => $apiResponseArr];
                    break;

                default:
                    $apiResponseArr = ['response_message' => 'The request couldn\'t be completed! Please try again later'];
                    return ['status' => false, 'api_response' => $apiResponseArr];
                    break;
            }
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    public function newProspect($formData)
    {
        $method                   = 'NewProspect';
        $configData               = [];
        $configData['campaignId'] = $this->campaignId;
        $configData['ipAddress']  = $this->ipAddress;
        $configData['offer_id']   = $this->offer_id;
        $configData['fname']      = $formData['firstName'];
        $configData['lname']      = $formData['lastName'];
        $crmData                  = array_merge_recursive($formData, $configData);
        $crmData                  = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $crmData                  = array_filter($crmData);
        $response                 = $this->callApi($method, $crmData);
        return $response;
    }

    public function newOrderWithProspect($checkoutData)
    {
        $method     = 'NewOrderWithProspect';
        $configData = $result     = array();
        foreach ($this->selectedProducts as $key => $selectedProduct)
        {
            $result[$key]['item_id']  = $selectedProduct['crm_product_id'];
            $result[$key]['quantity'] = $selectedProduct['crm_product_qty'];
            $result[$key]['price']    = $selectedProduct['crm_product_price'];
        }
        $configData['items']      = $result;
        $configData['campaignId'] = $this->campaignId;
        $configData['offer_id']   = $this->offer_id;
        $checkoutData['cc_type']  = $this->cardTypeId($checkoutData['cc_type']);
        if (strtolower($checkoutData['billingSameAsShipping']) !== 'yes')
        {
            $checkoutData                  = $this->prepareBillingDetails($checkoutData, $method);
            $checkoutData['shipping_same'] = false;
        }
        else
        {
            $checkoutData['shipping_same'] = true;
        }
        $crmData  = array_merge_recursive($checkoutData, $configData);
        $crmData  = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
        $response = $this->callApi($method, $crmData);
        return $response;
    }

    public function prepareBillingDetails($data, $method)
    {
        return Helper::renameArrayKey($data, self::$formKeys[$method]);
    }

    public function newOrder($orderData, $configData, $type)
    {
        $method             = 'NewOrder';
        $checkoutData       = $orderData;
        unset($orderData['billingSameAsShipping'], $orderData['cc_type'], $orderData['cc_number'], $orderData['expmonth'], $orderData['expyear'], $orderData['cvv'], $orderData['current_url'], $orderData['next_url'], $orderData['offer_id']);
        $prospectCreateCall = $this->newProspect($orderData);
        if ($prospectCreateCall['status'])
        {
            $data   = $result = array();
            foreach ($this->selectedProducts as $key => $selectedProduct)
            {
                $result[$key]['item_id']  = $selectedProduct['crm_product_id'];
                $result[$key]['quantity'] = $selectedProduct['crm_product_qty'];
                $result[$key]['price']    = $selectedProduct['crm_product_price'];
            }
            $data['items']                        = $result;
            $data['campaignId']                   = $this->campaignId;
            $data['offer_id']                     = $this->offer_id;
            $data['customers_address_billing_id'] = $prospectCreateCall['api_response']['customerAddressId'];
            $data['customer_id']                  = $prospectCreateCall['api_response']['prospectId'];
            $checkoutData['cc_type']              = $this->cardTypeId($checkoutData['cc_type']);
            unset($checkoutData['firstName'], $checkoutData['lastName'], $checkoutData['address'], $checkoutData['city'], $checkoutData['country'], $checkoutData['state'], $checkoutData['zip'], $checkoutData['phone'], $checkoutData['email']);
            if (strtolower($checkoutData['billingSameAsShipping']) !== 'yes')
            {
                $checkoutData                  = $this->prepareBillingDetails($checkoutData, $method);
                $checkoutData['shipping_same'] = false;
            }
            else
            {
                $checkoutData['shipping_same'] = true;
            }
            $crmData  = array_merge_recursive($checkoutData, $data);
            $crmData  = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
            $response = $this->callApi($method, $crmData);
            return $response;
        }
        return $prospectCreateCall;
    }

    public function orderView($queryStrings, $orderIds)
    {
        $method                          = 'orderView';
        $crmData['order_id']             = implode(',', $orderIds);
        $api                             = $this->prepareEndpointApi($this->api_base_url, $method);
        $form_post_params['form_params'] = [
            'user_id'       => $this->username,
            'user_password' => urlencode($this->password),
        ];
        $api                             = $api . $crmData['order_id'] . '?' . http_build_query($form_post_params['form_params']);
        $response                        = $this->httpClient->get($api);
        $httpResponse                    = $response->getStatusCode();
        if ($httpResponse == 200 || $httpResponse == 400)
        {
            $apiResponse = $response->getBody()->getContents();
            $apiResponse = json_decode($apiResponse, true);
            if (!empty($apiResponse['data']))
            {
                return $apiResponse['data'];
            }
            else
            {
                return $apiResponse['message'];
            }
        }
    }

    private function perform3DSActions($method, $apiResponse, $form_post_params)
    {
        $_SESSION['3ds_meta']['crm_data'] = array(
            'method'              => $method,
            'next_url'            => $form_post_params['next_url'],
            '3ds_post_url'        => $apiResponse['data']['transaction']['3ds_post_url'],
            'gateway_response_id' => $apiResponse['data']['transaction']['gateway_response_id'],
            'merchant_id'         => $apiResponse['data']['transaction']['merchant_id'],
            'transaction_id'      => $apiResponse['data']['transaction']['transaction_id'],
            'customer_id'         => $apiResponse['data']['transaction']['customer_id'],
            'order_id'            => $apiResponse['data']['transaction']['order_id'],
            'current_url'         => $form_post_params['current_url'],
        );
        $data                             = array(
            'billing-cc-number' => $form_post_params['card_number'],
            'billing-cvv'       => $form_post_params['card_cvv'],
            'billing-cc-exp'    => $form_post_params['card_exp_month'] . $form_post_params['card_exp_year']
        );
        header('Content-Type: application/json');
        echo json_encode(array(
            'status'    => 'success', 'is_html'   => true,
            'html_data' => $this->generate3DSForm($apiResponse['data']['transaction']['3ds_post_url'], $data),
        ));
        exit;
    }

    public function process3DOrder($token, $crmData)
    {
        $method                             = 'process3ds';
        $api                                = $this->prepareEndpointApi($this->api_base_url, $method);
        $form_params                        = [
            'user_id'               => $this->username,
            'user_password'         => urlencode($this->password),
            'transaction_id'        => $crmData['transaction_id'],
            'transaction_3ds_token' => $token,
        ];
        $response                           = $this->httpClient->post($api, ['body' => $form_params]);
        $httpResponse                       = $response->getStatusCode();
        $apiResponse                        = $response->getBody()->getContents();
        $apiResponse                        = json_decode($apiResponse, true);
        $apiResponseArr['response_message'] = $apiResponse['message'];
        switch ($httpResponse)
        {
            case 200:
                $apiResponseArr = [
                    'orderId'    => $_SESSION['3ds_meta']['crm_data']['order_id'],
                    'customerId' => $_SESSION['3ds_meta']['crm_data']['customer_id'],
                    'prospectId' => $_SESSION['3ds_meta']['crm_data']['customer_id'],
                ];
                return ['status' => $apiResponse['success'], 'api_response' => $apiResponseArr];
                break;

            default:
                return ['status' => $apiResponse['success'], 'api_response' => $apiResponseArr];
                break;
        }
    }

    protected function generate3DSForm($url, $data)
    {
        $form = <<<HTML
            <html>
                <head></head>
                <body onload="document.threeDForm.submit()" >
                    <form name="threeDForm" action="{$url}" id="threeDForm" method="POST">
HTML;
        foreach ($data as $key => $value)
        {
            $form .= <<<HTML
                        <input type="hidden" name="{$key}" value="{$value}">
HTML;
        }
        $form .= <<<HTML
                    </form>
                </body>
            </html>
HTML;
        return $form;
    }

    public function isDecline($response)
    {
        
    }

    public function cancelOrder($formData)
    {
        
    }

    public function singleOrderView($formData)
    {
        
    }

    public function hasOwnedThisOrder($data, $formData)
    {
        
    }

    public function prepareDataForCancel($data, $formData)
    {
        
    }

}
