<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;

class Stripe extends Crm implements CrmContracts
{
    protected $endPoint = 'https://api.stripe.com/v1';
    protected $apiPublicKey;
    protected $apiSecrectKey;
    protected $httpVerb;

    public function __construct()
    {
        parent::__construct();
        $this->apiPublicKey   = Config::get('payment_provider.api_public_key');
        $this->apiSecrectKey  = Config::get('payment_provider.api_secret_key');
    }
    
    /**
    * Call API 
    * @param string method
    * @param array form_post_params
    * @return array
    */
    protected function callApi($method, $form_post_params)
    {
        
        $headers[] = 'Authorization: Bearer '.$this->apiSecrectKey;
        $headers[]  = 'Content-Type: application/x-www-form-urlencoded';
        $api        = $this->endPoint.'/'.$method;

        $ch                = curl_init();
        curl_setopt($ch, CURLOPT_URL, $api);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $this->httpVerb);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        
        if($this->httpVerb === 'POST')
        {
            curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($form_post_params));
        }
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $result = curl_exec($ch);
        if (curl_errno($ch))
        {
            echo 'Error:' . curl_error($ch);
        }
        curl_close($ch);
        $response = json_decode($result, 1);
       
        return $response;
    }


    /**
    * Prepare payload for prospect
    * @param array formData
    * @return array
    */
    public function newProspect($formData)
    {
        $method     = 'customers';
        $configData = [];
            
        $shippingData = $this->prepareShippingDetails($formData);

        $configData['phone'] = $formData['phone'];
        $configData['email'] = $formData['email'];
        $crmData = array_merge_recursive($configData, $shippingData);
                      
        $this->httpVerb = 'POST';
        $response = $this->callApi($method, $crmData);

        return $this->prepareGatewayMessage($response,'prospect');
       
    }

    /**
    * Prepare payload for new order with prospect
    * @param array formData
    * @return array
    */
    public function newOrderWithProspect($formData)
    {
      
        $method                      = 'payment_intents';
        $shippingData                = $this->prepareShippingDetails($_SESSION['prospect_shipping_data']);
        $productPrice                 = $this->prepareProductDetails();
        
        if (strtolower($formData['billingSameAsShipping']) !== 'yes')
        {
            $billingData   = $this->prepareBillingDetails($formData);
        }
       
        $billingData['billing']['name'] = $_SESSION['prospect_shipping_data']['firstName'].' '.$_SESSION['prospect_shipping_data']['lastName'];
        $billingData['billing']['email'] = $_SESSION['prospect_shipping_data']['email'];
        $billingData['billing']['phone'] = $_SESSION['prospect_shipping_data']['phone'];
      
        $cardInfoData['card']           = $this->prepareCardDetails($formData);
        $tokenInfo                      = $this->generateToken($billingData['billing'],$cardInfoData['card']);

        if(!$tokenInfo['status'])
        {
            return $tokenInfo;
        }

        $attachToken = $this->attachTokenCustomer($tokenInfo['api_response']['tokenId'],$formData['prospectId']);
        
        if(!$attachToken['status'])
        {
            return $attachToken;
        }

        $payload['amount']                 = $productPrice;
        $payload['currency']               = 'usd';
        $payload['payment_method_types']   = ['card'];
        $payload['payment_method']         = $attachToken['api_response']['paymentId'];
        $payload['customer']               = $formData['prospectId'];
        $payload['receipt_email']          = $_SESSION['prospect_shipping_data']['email'];
        $payload['shipping']               = $shippingData['shipping'];
        $payload['shipping']['phone']      = $_SESSION['prospect_shipping_data']['phone'];
        $payload['confirm']                = "true";

        $this->httpVerb = 'POST';

        $response       = $this->callApi($method, $payload);
        
        $responseMsg    = $this->prepareGatewayMessage($response,'order');
        if(!empty($responseMsg['status']))
        {
            $responseMsg['api_response']['customerId'] = $formData['prospectId'];
        }
        return $responseMsg;
       
    }

    /**
    * Prepare payload for new order
    * @param array formData
    * @return array
    */
    public function newOrder($formData, $configData, $type)
    {
        $method                      = 'payment_intents';
        $getCustomer                 = $this->newProspect($formData);

        if(!$getCustomer['status'])
        {
            return $getCustomer;
        }
        $shippingData    = $this->prepareShippingDetails($formData);
        $productPrice    = $this->prepareProductDetails();
        
        if (strtolower($formData['billingSameAsShipping']) !== 'yes')
        {
            $billingData   = $this->prepareBillingDetails($formData);
        }
       
        $billingData['billing']['name']  = $formData['firstName'].' '.$formData['lastName'];
        $billingData['billing']['email'] = $formData['email'];
        $billingData['billing']['phone'] = $formData['phone'];

        $cardInfoData['card']           = $this->prepareCardDetails($formData);
        $tokenInfo                      = $this->generateToken($billingData['billing'],$cardInfoData['card']);
        
        if(!$tokenInfo['status'])
        {
            return $tokenInfo;
        }
        
        $attachToken = $this->attachTokenCustomer($tokenInfo['api_response']['tokenId'],$getCustomer['api_response']['prospectId']);
        if(!$attachToken['status'])
        {
            return $attachToken;
        }
          
        $payload['amount']                 = $productPrice;
        $payload['currency']               = 'usd';
        $payload['payment_method_types']   = ['card'];
        $payload['payment_method']         = $attachToken['api_response']['paymentId'];
        $payload['customer']               = $getCustomer['api_response']['prospectId'];
        $payload['receipt_email']          = $_SESSION['prospect_shipping_data']['email'];
        $payload['shipping']               = $shippingData['shipping'];
        $payload['shipping']['phone']      = $_SESSION['prospect_shipping_data']['phone'];
        $payload['confirm']                = "true";
       
        $this->httpVerb = 'POST';
        $response = $this->callApi($method, $payload);
        
        $responseMsg = $this->prepareGatewayMessage($response,'order');
        if(!empty($responseMsg['status']))
        {
            $responseMsg['api_response']['customerId'] = $getCustomer['api_response']['prospectId'];
        }
        
        return $responseMsg;
    }  

    /**
    * Prepare payload for order view
    * @param string queryStrings
    * @param array orderIds
    * @return array
    */
    public function orderView($queryStrings, $orderIds)
    {
        $method              = 'payment_intents/'.$orderIds[0];
        $this->httpVerb      = 'GET';
        $response = $this->callApi($method, []);
        
        if (!empty($response) && (!empty($response['id'])))
        {
            foreach($this->selectedProducts as $offerKey => $product) 
            {
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['product_id']           = $product['crm_product_id'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['name']        = $product['product_name'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['price']       = $product['product_price'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['product_qty'] =$product['product_qty'];
           
            }
      
            $apiResponseArr['order_total'] = $response['amount'];
        }
        else
        {
            $apiResponseArr = $response;
        }
       
        return $apiResponseArr;
    }

    /**
    * Get Product Info
    * @param array $formData
    * @return array
    */
    private function prepareProductDetails() 
    {

        $product_price = 0.00;
        foreach($this->selectedProducts as $product) 
        {
            $qty           = empty($product['crm_product_qty']) ? 1 : $product['crm_product_qty'];
            $product_price = $product_price + ($qty * $product['crm_product_price']);
           
        }
        return ($product_price+$this->shippingProfile->get('shipping_profile_price')) * 100;
    }

    /**
    * Preparing Shipping Details
    * @param array $formData
    * @return array
    */
    private function prepareShippingDetails($formData)
    {
        $shippingDetails = array(
            'line1'        => $formData['address'],
            'postal_code'  => $formData['zip'],
            'city'         => $formData['city'],
            'state'        => $formData['state'],
            'country'      => $formData['country'],
        );

        $configData['shipping']['address'] = $shippingDetails;
        $configData['shipping']['name'] = $formData['firstName'].' '.$formData['lastName'];
        return $configData;
    }

    /**
    * Preparing Billing Details
    * @param array $formData
    * @return array
    */
    private function prepareBillingDetails($formData)
    {
        $configData = [];
        if (strtolower($formData['billingSameAsShipping']) !== 'yes')
        {
            $billingAddress = array(
                'line1'      => $formData['billingAddress'],
                'postal_code'=> $formData['billingZip'],
                'city'       => $formData['billingCity'],
                'state'      => $formData['billingState'],
                'country'    => $formData['billingCountry']
            );
            $configData['billing']['address'] = $billingAddress;
        }
        
        return $configData;
    }

    /**
    * Preparing Card Details
    * @param array $formData
    * @return array
    */
    private function prepareCardDetails($formData)
    {
        $paymentDetails = array(
            'number'    => $formData['cc_number'],
            'cvc'       => $formData['cvv'],
            'exp_month' => $formData['expmonth'],
            'exp_year'  => $formData['expyear']
        );
        return $paymentDetails;
    }

    /**
    * Generate Token
    * @param array $billingInfo
    * @return array $cardInfo
    */
    private function generateToken($billingInfo, $cardInfo)
    {
        $method = 'payment_methods';
        $tokenInfo = [
            'type' => 'card',  
            'card' => $cardInfo,
            'billing_details' => $billingInfo
        ];
        
        $this->httpVerb = 'POST';
        $response = $this->callApi($method, $tokenInfo);

        return $this->prepareGatewayMessage($response,'token');
    }

    /**
    * Attach token customer
    * @param string $tokenId
    * @return string $customerId
    */
    private function attachTokenCustomer($tokenId, $customerId)
    {
        $method = 'payment_methods/'.$tokenId.'/attach';
        $attachToken = [
            'customer' => $customerId
        ];
        $this->httpVerb = 'POST';
        $response = $this->callApi($method, $attachToken);
      
        return $this->prepareGatewayMessage($response,'payment');
        
    }

    /**
    * Prepare Message
    * @param string $gatewayMesage
    */
    private function prepareGatewayMessage($gatewayMesage,$type='')
    {
        $apiTypeArr = [
            'prospect' =>'prospectId',
            'order'    =>'orderId',
            'token'    =>'tokenId',
            'payment'  =>'paymentId'
        ];

        if (!empty($gatewayMesage) && (!empty($gatewayMesage['error']['message'])))
        {
            $status      = false;
            $arrayResult = array(
                'status' => false,
                'errors' => $gatewayMesage['error']['message'],
            );

        }else if (!empty($gatewayMesage) && (!empty($gatewayMesage['id']))){
            $status      = true;
            $arrayResult = array(
                'status'     => true
            );
            if(!empty($type))
            {
                $arrayResult[$apiTypeArr[$type]] = $gatewayMesage['id'];
            }

        }else{
            $status      = false; 
            $arrayResult = 'Something went wrong while processing the request';
        }

        return ['status'=>$status, 'api_response'=>$arrayResult];
    }


    /**
     * @param array $formData
     */
    public function singleOrderView($formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function hasOwnedThisOrder($responseData, $formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function prepareDataForCancel($responseData, $formData)
    {
        
    }

    /**
     * @param array $formData
     */
    public function cancelOrder($formData)
    {
        
    }

    /**
     * @param array $response
     */
    public function isDecline($response)
    {
        
    }

    /**
     * StripeGoogle Pay Btn implementation
     */
    public function paymentIntentsSecret($formData, $paymentIntentId)
    {
        $method         = empty($paymentIntentId) ? 'payment_intents' : 'payment_intents/'.$paymentIntentId;
        $total_price    = $this->prepareProductDetails();

        if($total_price <= 0.5){
            $token['status']        = false;
            $token['message']       = 'Total amount must be greater than $0.5';
            $token['total_price']   = $total_price;
            $token['stripe']['publishableKey']	= '';
            $token['stripe']['client_secret']   = '';
        }
        else{
            $params = [
                'amount'=> $total_price,
                'currency'=> strtolower(Config::get('funnel.currency.code')),
                'payment_method_types'=> ['card']
            ];
            $this->httpVerb = 'POST';
            $response = $this->callApi($method, $params);
            $token['total_price'] = $total_price;
            $token['stripe']['publishableKey']	= $this->apiPublicKey;
            $token['stripe']['client_secret']   = $response['client_secret'];
        }
        return json_encode($token);
    }
}
