<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;

class TwentyNineNext extends Crm implements CrmContracts
{

    protected $api_base_url;
    protected $apiKey;
    protected $selectedProducts;
    protected $httpVerb;
    protected $dialCode;

    protected static $formKeys          = [
        'NewProspect'          => [
            'phone'      => 'phone_number',
            'firstName'  => 'first_name',
            'lastName'   => 'last_name',
        ]
    ];


    public function __construct()
    {
        parent::__construct();
        $this->apiKey   = Config::get('payment_provider.api_password');
        $this->dialCode = Config::get('domain.support_dialing_code', '+1');
    }

    /**
    * Preparing Endpoint API
    * @param string api_base_url
    * @param string method
    * @return string
    */
    protected function prepareEndpointApi($api_base_url, $method)
    {
        $methodSlugMapping = [
            'NewProspect'          => 'users',
            'NewOrderWithProspect' => 'orders',
            'NewOrder'             => 'orders',
        ];
        if (array_key_exists($method, $methodSlugMapping))
        {
            $endpointUrl = preg_replace("(^https?://)", "", sprintf('%s/%s/%s/', rtrim($api_base_url, '/'), 'api/admin', $methodSlugMapping[$method]));
            $endpointUrl = 'https://'. $endpointUrl;
        }
        else
        {
            throw new \Exception("Invalid method name : {$method}", 9999);
        }
        return $endpointUrl;
    }

    /**
    * Call API 
    * @param string method
    * @param array form_post_params
    * @return array
    */
    protected function callApi($method, $form_post_params)
    {
        
        $headers[] = 'Authorization: Token ' . $this->apiKey;
        $headers[]  = 'Content-Type: application/json';
        if($this->httpVerb === 'GET'){
            $api = rtrim($this->api_base_url.'/api/admin/'.$method.'/');
        }else{
            $api = $this->prepareEndpointApi($this->api_base_url, $method);
        }
        
        $ch                = curl_init();
        curl_setopt($ch, CURLOPT_URL, $api);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $this->httpVerb);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
          
        if($this->httpVerb === 'POST'){
            
            curl_setopt($ch, CURLOPT_POSTFIELDS, $form_post_params);
        }
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

        $result = curl_exec($ch);
        if (curl_errno($ch))
        {
            echo 'Error:' . curl_error($ch);
        }
        curl_close($ch);
        $response = json_decode($result, 1);

        return $response;
    }


    /**
    * Prepare payload for prospect
    * @param array formData
    * @return array
    */
    public function newProspect($formData)
    {
        try
        {
            $method     = 'NewProspect';
            $configData = [];
            $configData['ip']             = $this->ipAddress;
            $configData['phone_number']   = $this->dialCode.$formData['phone'];
            $shippingData['addresses'][0] = $this->prepareShippingDetails($formData);
            
            $crmData = array_merge_recursive($formData, $configData, $shippingData);
            $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
            $crmData = array_filter($crmData);
             unset(
                $crmData['address'], $crmData['city'], $crmData['country'], $crmData['state'], $crmData['zip']
            );

            $this->httpVerb = 'POST';

            $response = $this->callApi($method, json_encode($crmData));

            if (!empty($response) && (empty($response['id'])))
            {
                $status      = false;
                $arrayResult = array(
                    'status' => false,
                    'errors' => $response,
                );

            }else if (!empty($response) && (!empty($response['id']))){

                $status      = true;
                $arrayResult = array(
                    'status'     => true,
                    'prospectId' => $response['id'],
                );

            }else{
                $status = false; 
                $arrayResult = 'Something went wrong while processing the request';
            }

            return ['status' => $status, 'api_response' => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => APP_DEBUG ? $ex->getMessage() : 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    /**
    * Prepare payload for new order with prospect
    * @param array formData
    * @return array
    */
    public function newOrderWithProspect($formData)
    {
        try
        {
            $method                               = 'NewOrderWithProspect';
            $configData['payment_method']         = 'bankcard';

            $shippingData['shipping_address']     = $this->prepareShippingDetails($_SESSION['prospect_shipping_data']);
            if (strtolower($formData['billingSameAsShipping']) !== 'yes')
            {
                $billingData['billing_address']    = $this->prepareBillingDetails($formData);
            }else{
                $billingData['billing_same_as_shipping_address']    = true;
                
            }
            $configData['payment_details']        = $this->prepareCardDetails($formData);
            $configData['payment_details']['card_name'] = $_SESSION['prospect_shipping_data']['firstName'].' '.$_SESSION['prospect_shipping_data']['lastName'];
            $configData['lines'] = $this->prepareProductDetails($formData);
            $crmData = array_merge_recursive($formData, $configData, $shippingData, $billingData);
            $crmData = array_filter($crmData);
            $crmData['user']['id'] = $crmData['prospectId'];
                
            unset($crmData['cc_type'],$crmData['productQty'],$crmData['productId'],$crmData['prospectId'],$crmData['billingSameAsShipping'],$crmData['prospectId'],$crmData['billingFirstName'],$crmData['billingLastName'],$crmData['billingAddress'],$crmData['billingCity'],$crmData['billingCountry'],$crmData['billingState'],$crmData['billingZip'],$crmData['cc_number'],$crmData['expmonth'],$crmData['expyear'],$crmData['cvv'],$crmData['name_on_card']);
            
            $this->httpVerb = 'POST';
            $response = $this->callApi($method, json_encode($crmData));
            
            if (!empty($response) && (!empty($response['number'])))
            {
                $status      = true;
                $arrayResult['orderId'] = $response['number'];
                $arrayResult['customerId'] = $response['user']['id'];
            }
            else if (!empty($response) && empty($response['number']))
            {
                $status      = false;
                $arrayResult = array(
                    'status' => false,
                    'errors' => $response,
                );
            }else{
                $status = false; 
                $arrayResult = 'Something went wrong while processing the request';
            }
            
            return ['status' => $status, 'api_response' => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    /**
    * Prepare payload for new order
    * @param array formData
    * @return array
    */
    public function newOrder($formData, $configData, $type)
    {
        try
        {
            $method                               = 'NewOrder';
            $configData['payment_method']         = 'bankcard';
            $configData['attribution']            = $this->prepareAttributeDetails();
            $configData['user']['email']          = $formData['email'];
            $configData['user']['first_name']     = $formData['firstName'];
            $configData['user']['last_name']      = $formData['lastName'];
            $configData['user']['ip']             = $this->ipAddress;
            $configData['user']['phone_number']   = $this->dialCode.$formData['phone'];
            $shippingData['shipping_address']     = $this->prepareShippingDetails($_SESSION['prospect_shipping_data']);
            if (strtolower($formData['billingSameAsShipping']) !== 'yes')
            {
                $billingData['billing_address']    = $this->prepareBillingDetails($formData);
            }else{
                $billingData['billing_same_as_shipping_address']    = true;
                
            }
            $configData['payment_details']        = $this->prepareCardDetails($formData);
            $configData['payment_details']['card_name'] = $formData['firstName'].' '.$formData['lastName'];
            $configData['lines'] = $this->prepareProductDetails($formData);
            $crmData = array_merge_recursive($formData, $configData, $shippingData, $billingData);
            $crmData = array_filter($crmData);
                
            unset($crmData['cc_type'],$crmData['firstName'],$crmData['lastName'],$crmData['address'],$crmData['billingSameAsShipping'],$crmData['city'],$crmData['country'],$crmData['state'],$crmData['zip'],$crmData['phone'],$crmData['email'],$crmData['cc_number'],$crmData['selectedProducts'],$crmData['campaignId'],$crmData['expmonth'],$crmData['expyear'],$crmData['cvv'],$crmData['name_on_card'],$crmData['shippingId']);
            
            $this->httpVerb = 'POST';
              
            $response = $this->callApi($method, json_encode($crmData));
            
            if (!empty($response) && (!empty($response['number'])))
            {
                $status      = true;
                $arrayResult['orderId']    = $response['number'];
                $arrayResult['customerId'] = $response['user']['id'];
            }
            else if (!empty($response) && empty($response['number']))
            {
                $status      = false;
                $arrayResult = array(
                    'status' => false,
                    'errors' => $response,
                );
            }else{
                $status = false; 
                $arrayResult = 'Something went wrong while processing the request';
            }
            
            return ['status' => $status, 'api_response' => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }  

    /**
    * Prepare payload for order view
    * @param string queryStrings
    * @param array orderIds
    * @return array
    */
    public function orderView($queryStrings, $orderIds)
    {
        $method              = 'orderView';
        $this->httpVerb      = 'GET';
        $response = $this->callApi('orders/'.$orderIds[0], []);
        
        if (!empty($response) && (!empty($response['number'])))
        {
            foreach ($response['lines'] as $offerKey => $offerInfo) 
            {
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['product_id'] = $offerInfo['stockrecord']['partner_sku'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['name']       = $offerInfo['product']['title'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['price']      = $offerInfo['price_incl_tax'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['product_qty'] = $offerInfo['quantity'];
                
            }
            $apiResponseArr['order_total'] = $response['total_incl_tax'];
        }
        else
        {
            $apiResponseArr = $response;
        }
        
        return $apiResponseArr;
    }

    /**
    * Get Product Info
    * @param array $formData
    * @return array
    */
    private function prepareProductDetails($formData) 
    {
        $offers = [];
        $products = [];
        foreach($this->selectedProducts as $product) 
        {
            $products = [
                'sku' => $product['product_sku'],
                'quantity' => array_get($product, 'crm_product_qty', '1'),
            ];
            if(!empty($product['crm_product_price']))
            {
                $products['price_excl_tax'] = $product['crm_product_price'];
            }

            array_push($offers, $products);
        }

        return $offers;
    }

    /**
    * Preparing Shipping Details
    * @param array $formData
    * @return array
    */
    private function prepareShippingDetails($formData) 
    {
        $configData['line1']        = $formData['address'];
        $configData['line2']        = '';
        $configData['line4']        = $formData['city'];
        $configData['country']      = $formData['country'];
        $configData['state']        = $formData['state'];
        $configData['postcode']     = $formData['zip'];
        $configData['first_name']   = $formData['firstName'];
        $configData['last_name']    = $formData['lastName'];
        $configData['phone_number'] = $this->dialCode.$formData['phone'];

        return $configData;
    }

    /**
    * Preparing Billing Details
    * @param array $formData
    * @return array
    */
    private function prepareBillingDetails($formData)
    {
        $billingAddress = [];
        $billingAddress = array(
            'first_name' => $formData['billingFirstName'],
            'last_name'  => $formData['billingLastName'],
            'line1'      => $formData['billingAddress'],
            'postcode'  => $formData['billingZip'],
            'line4'     => $formData['billingCity'],
            'state'     => $formData['billingState'],
            'country'   => $formData['billingCountry']
        );
        return $billingAddress;
    }

    /**
    * Preparing Payment Method
    * @param array $formData
    * @return array
    */
    private function prepareCardDetails($formData)
    {
        $paymentDetails = [];
        $paymentDetails = array(
            'card_number'  => $formData['cc_number'],
            'card_cvv'      => $formData['cvv'],
            'card_expiry_month'  => $formData['expmonth'],
            'card_expiry_year'     => $formData['expyear'],
            'save_card'     => true,
            'payment_gateway'  => Config::get('offer_details.step1.crm_force_gateway_id')
        );
        return $paymentDetails;
    }

    /**
     * @param array $formData
     */
    public function singleOrderView($formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function hasOwnedThisOrder($responseData, $formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function prepareDataForCancel($responseData, $formData)
    {
        
    }

    /**
     * @param array $formData
     */
    public function cancelOrder($formData)
    {
        
    }

    /**
     * @param array $response
     */
    public function isDecline($response)
    {
        
    }

    private function prepareAttributeDetails(){
        $AttributeDetails = ['funnel' => Config::get('funnel.name')];
        return $AttributeDetails;
    }
}
