<?php

namespace Framework\Modules\Crm\Providers;

use Framework\Foundation\Config;
use Framework\Modules\Crm\Contracts\Crm as CrmContracts;
use Framework\Foundation\Helper;

class Vrio extends Crm implements CrmContracts
{

    protected $api_base_url;
    protected $username;
    protected $campaignId;
    protected $connectionId;
    protected $selectedProducts;
    protected $ipAddress;
    protected $httpVerb;
    protected $end_point_param;
    

    protected static $formKeys          = [
        'NewProspect'          => [
            'address'    => 'bill_address1',
            'address2'   => 'bill_address2',
            'city'       => 'bill_city',
            'country'    => 'bill_country',
            'state'      => 'bill_state',
            'zip'        => 'bill_zipcode',
            'firstName'  => 'bill_fname',
            'lastName'   => 'bill_lname',
        ],
        'NewOrderWithProspect' => [
            'cc_number'        => 'card_number',
            'cvv'              => 'card_cvv',
            'expmonth'         => 'card_exp_month',
            'expyear'          => 'card_exp_year',
            'billingFirstName' => 'bill_fname',
            'billingLastName'  => 'bill_lname',
            'billingCountry'   => 'bill_country',
            'billingAddress'   => 'bill_address1',
            'billingCity'      => 'bill_city',
            'billingState'     => 'bill_state',
            'billingZip'       => 'bill_zipcode',
            'prospectId'       => 'order_id',
            'billingSameAsShipping'=>'shipping_same'
        ],
        'NewOrder'             => [
            'cc_number'        => 'card_number',
            'cvv'              => 'card_cvv',
            'expmonth'         => 'card_exp_month',
            'expyear'          => 'card_exp_year',
            'billingFirstName' => 'bill_fname',
            'billingLastName'  => 'bill_lname',
            'billingCountry'   => 'bill_country',
            'billingAddress'   => 'bill_address1',
            'billingCity'      => 'bill_city',
            'billingState'     => 'bill_state',
            'billingZip'       => 'bill_zipcode',
            'billingSameAsShipping'=>'shipping_same'
        ],
    ];


    public function __construct()
    {
        parent::__construct();
        $this->connectionId = Config::get('offer_details.step1.connection_id');
        $this->forceGatewayId = Config::get('offer_details.step1.crm_force_gateway_id');
    }

    /**
    * Preparing Endpoint API
    * @param string api_base_url
    * @param string method
    * @return string
    */
    protected function prepareEndpointApi($api_base_url, $method)
    {
        $methodSlugMapping = [
            'NewProspect'          => 'orders',
            'NewOrderWithProspect' => 'orders',
            'NewOrder'             => 'orders',
            'orderView'            => 'orders',
        ];
        if (array_key_exists($method, $methodSlugMapping))
        {
            $endpointUrl = $api_base_url . '/' . $methodSlugMapping[$method];
        }
        else
        {
            throw new \Exception("Invalid method name : {$method}", 9999);
        }
        return $endpointUrl;
    }

    /**
    * Call API 
    * @param string method
    * @param array form_post_params
    * @return array
    */
    protected function callApi($method, $form_post_params)
    {
        $processApis = ['NewOrderWithProspect','NewOrder'];
        if(in_array($method,$processApis)){
            $form_post_params['action'] = 'process';
        }

        if($this->httpVerb === 'GET'){
            $api = rtrim($this->api_base_url.'/'.$method, '/') . '?' . http_build_query($form_post_params);
        }else{
            $api = $this->prepareEndpointApi($this->api_base_url, $method);
        }


        $headers = [
            "X-Api-Key: $this->username",
            "accept: application/json",
            "content-type: application/json",
            "hostname: ".str_replace(['https://','/'],['',''],$this->api_base_url)
          ];

        $curl = curl_init();

        curl_setopt_array($curl, [
          CURLOPT_URL => $api,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 30,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => "POST",
          CURLOPT_HTTPHEADER => $headers,
          CURLOPT_POSTFIELDS => json_encode($form_post_params)
        ]);

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
          $message = "Something went wrong while processing the request";            
          $message .= " cURL exception | file name Framework/Modules/Crm/Providers/Vrio.php, Line 144 :" . $err;
          $code = AJAX_EX_CODE;
          throw new \Exception($message, $code);          
        } else {
            return json_decode($response,1);
        }

    }



    /**
    * Prepare payload for prospect
    * @param array formData
    * @return array
    */
    public function newProspect($formData)
    {
        try
        {
            $method     = 'NewProspect';
            $configData = [];
            $configData['campaign_id']    = $this->campaignId;
            $configData['ip_address']     = $this->ipAddress;
            $configData['connection_id']  = $this->connectionId;
            $configData['shipping_same']  = true;
            $shippingData = $this->prepareShippingDetails($formData);
         
            $crmData = array_merge_recursive($formData, $configData, $shippingData);
 
            $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
            $crmData = array_filter($crmData);

            $this->httpVerb = 'POST';
            $response = $this->callApi($method, $crmData);

            if (!empty($response) && (!empty($response['success'])))
            {
                $status      = true;
                $arrayResult = array(
                    'status'     => true,
                    'prospectId' => $response['order_id'],
                );
            }
            else if (!empty($response) && !empty($response['error']))
            {
                $status      = false;
                $arrayResult = array(
                    'status' => false,
                    'errors' => $response['error']['message'],
                );
            }else{
                $status = false; 
                $arrayResult = 'Something went wrong while processing the request';
            }

            return ['status' => $status, 'api_response' => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => (APP_DEBUG || $ex->getCode() === AJAX_EX_CODE) ? $ex->getMessage() : 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    /**
    * Prepare payload for new order with prospect
    * @param array formData
    * @return array
    */
    public function newOrderWithProspect($formData)
    {
        try
        {
            $method                               = 'NewOrderWithProspect';
            $configData['campaign_id']            = $this->campaignId;
            $configData['connection_id']          = $this->connectionId;
            $configData['payment_method_id']      = 1;
            $configData['shipping_profile_id']    = $this->shippingProfile->get('shipping_profile_crm_id');
        
            $configData['phone'] = $_SESSION['prospect_shipping_data']['phone'];
            $configData['email'] = $_SESSION['prospect_shipping_data']['email'];
        
            if($formData['billingSameAsShipping']=='yes')
            {
                $configData['bill_fname']    = $_SESSION['prospect_shipping_data']['firstName'];
                $configData['bill_lname']    = $_SESSION['prospect_shipping_data']['lastName'];
                $configData['bill_country']  = $_SESSION['prospect_shipping_data']['country'];
                $configData['bill_address1'] = $_SESSION['prospect_shipping_data']['address'];
                $configData['bill_city']     = $_SESSION['prospect_shipping_data']['city'];
                $configData['bill_state']    = $_SESSION['prospect_shipping_data']['state'];
                $configData['bill_zipcode']  = $_SESSION['prospect_shipping_data']['zip'];
                $shippingData                = [];
            }else{
                $shippingData = $this->prepareShippingDetails($_SESSION['prospect_shipping_data']);
            }
            $configData['offers'] = $this->prepareProductDetails($formData);
            $crmData = array_merge_recursive($formData, $configData, $shippingData);
 
            $crmData = Helper::renameArrayKey($crmData, self::$formKeys[$method]);
            $crmData = array_filter($crmData);
            unset($crmData['cc_type']);
            unset($crmData['productQty']);
            unset($crmData['productId']);
            unset($crmData['name_on_card']);
            $crmData['card_type_id']              = $this->cardTypeId($formData['cc_type']);
            $crmData['offer_restrict'] = true;
            $this->httpVerb = 'POST';
            
            // Override default gateway / merchant id
            if(!empty($this->forceGatewayId)) {
                $orderDcrmDataata['merchant_id'] = $this->forceGatewayId;
            }

            $response = $this->callApi($method, $crmData);
       
            if (!empty($response) && (!empty($response['success'])))
            {
                $status      = true;
                $arrayResult['orderId'] = $response['order_id'];
                $arrayResult['customerId'] = $response['customer_id'];
            }
            else if (!empty($response) && !empty($response['error']['message']))
            {
                $status      = false;
                $arrayResult =  $response['error']['message'];
            }else{
                $status = false; 
                $arrayResult = 'Something went wrong while processing the request';
            }
            
            return ['status' => $status, 'api_response' => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => (APP_DEBUG || $ex->getCode() === AJAX_EX_CODE) ? $ex->getMessage() : 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }

    /**
    * Prepare payload for new order
    * @param array formData
    * @return array
    */
    public function newOrder($formData, $configData, $type)
    {
        try
        {
            $crmData = [];
            $method                       = 'NewOrder';
            $crmData['campaign_id']    = $this->campaignId;
            $crmData['ip_address']     = $this->ipAddress;
            $crmData['connection_id']  = $this->connectionId;
            $crmData['payment_method_id'] = 1;
            $crmData['shipping_profile_id'] = $this->shippingProfile->get('shipping_profile_crm_id');
        
            $crmData['offers'] = $this->prepareProductDetails($formData);
            if($formData['billingSameAsShipping']=='yes')
            {
                $crmData['bill_fname']    = $_SESSION['prospect_shipping_data']['firstName'];
                $crmData['bill_lname']    = $_SESSION['prospect_shipping_data']['lastName'];
                $crmData['bill_country']  = $_SESSION['prospect_shipping_data']['country'];
                $crmData['bill_address1'] = $_SESSION['prospect_shipping_data']['address'];
                $crmData['bill_city']     = $_SESSION['prospect_shipping_data']['city'];
                $crmData['bill_state']    = $_SESSION['prospect_shipping_data']['state'];
                $crmData['bill_zipcode']  = $_SESSION['prospect_shipping_data']['zip'];
                $shippingData                = [];
            }else{
                $shippingData = $this->prepareShippingDetails($_SESSION['prospect_shipping_data']);
            }
            
            $orderData = array_merge_recursive($formData, $crmData, $shippingData);
 
            $orderData = Helper::renameArrayKey($orderData, self::$formKeys[$method]);
            $orderData = array_filter($orderData);
            unset($orderData['cc_type']);
            unset($orderData['productQty']);
            unset($orderData['productId']);
            unset($orderData['name_on_card']);
            $orderData['card_type_id']   = $this->cardTypeId($formData['cc_type']);
            $orderData['offer_restrict'] = true;
            $this->httpVerb = 'POST';
               
            // Override default gateway / merchant id
            if(!empty($this->forceGatewayId)) {
                $orderData['merchant_id'] = $this->forceGatewayId;
            }

            $response = $this->callApi($method, $orderData);
            if (!empty($response) && (!empty($response['success'])))
            {
                $status      = true;
                $arrayResult['orderId'] = $response['order_id'];
                $arrayResult['customerId'] = $response['customer_id'];
            }
            else if (!empty($response) && !empty($response['error']['message']))
            {
                $status      = false;
                $arrayResult =  $response['error']['message'];
            }else{
                $status = false; 
                $arrayResult = 'Something went wrong while processing the request';
            }
            return ['status' => $status, 'api_response' => $arrayResult];
        }
        catch (\Exception $ex)
        {
            $apiResponseArr = ['response_message' => (APP_DEBUG || $ex->getCode() === AJAX_EX_CODE) ? $ex->getMessage() : 'Something went wrong while processing the request'];
            return ['status' => false, 'api_response' => $apiResponseArr];
        }
    }   

    /**
    * Prepare payload for order view
    * @param string queryStrings
    * @param array orderIds
    * @return array
    */
    public function orderView($queryStrings, $orderIds)
    {
        $method              = 'orders/'.$orderIds[0];
        $this->httpVerb      = 'GET';
        $response = $this->callApi($method, ['with'=>'order_offers', 'order_gift_cards', 'order_revisions', 'transactions', 'customer', 'customer_card', 'customer_address_billing', 'customer_address_shipping']);
        $orderTotal = 0.00;
        
        if (!empty($response['success']))
        {
            foreach ($response['order_offers'] as $offerKey => $offerInfo) 
            {
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['product_id'] = $offerInfo['offer_id'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['name']       = $offerInfo['offer_name'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['price']      = $offerInfo['order_offer_price'];
                $apiResponseArr[$orderIds[0]]['products'][$offerKey]['product_qty'] = $offerInfo['order_offer_quantity'];
                $orderTotal = $orderTotal + $offerInfo['order_offer_price'];
            }
             $apiResponseArr['order_total'] = $orderTotal;
        }
        else
        {
            $apiResponseArr = $response['error']['message'];
        }
        
        return $apiResponseArr;
    }

    /**
    * Get Card Type
    * @param string $key
    * @return string
    */
    private function cardTypeId($key)
    {
        $cardTypeID = ['master' => 1, 'mastercard' => 1, 'visa' => 2, 'discover' =>3];
        if (array_key_exists($key, $cardTypeID))
        {
            return $cardTypeID[$key];
        }
        else
        {
            throw new \Exception("Invalid card type : {$key}", 9999);
        }
    }

    /**
    * Get Product Info
    * @param array $formData
    * @return array
    */
    private function prepareProductDetails($formData) 
    {
        $offers = array();
        foreach($this->selectedProducts as $product) {
        array_push($offers, array(
            "offer_id"             => $product['crm_product_id'],
            "discount_code"        => !empty($formData['promo_code']) ? $formData['promo_code'] : '',
            //"order_offer_shipping" => !empty($this->shippingProfile->get('shipping_profile_price')) ? $this->shippingProfile->get('shipping_profile_price') : '',
            "order_offer_quantity" => $product['crm_product_qty'],
            "order_offer_price"    => $product['crm_product_price']
            ));
        }

        return $offers;
    }

    /**
    * Preparing Shipping Details
    * @param array $formData
    * @return array
    */
    private function prepareShippingDetails($formData) 
    {
        $configData['ship_address1']  = $formData['address'];
        $configData['ship_city']      = $formData['city'];
        $configData['ship_country']   = $formData['country'];
        $configData['ship_state']     = $formData['state'];
        $configData['ship_zipcode']   = $formData['zip'];
        $configData['ship_fname']     = $formData['firstName'];
        $configData['ship_lname']     = $formData['lastName'];

        return $configData;
    }

    /**
     * @param array $formData
     */
    public function singleOrderView($formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function hasOwnedThisOrder($responseData, $formData)
    {
        
    }

    /**
     * @param array $responseData
     * @param array $formData
     */
    public function prepareDataForCancel($responseData, $formData)
    {
        
    }

    /**
     * @param array $formData
     */
    public function cancelOrder($formData)
    {
        
    }

    /**
     * @param array $response
     */
    public function isDecline($response)
    {
        
    }

}
